# functions

shopt | grep compat31 >/dev/null && shopt -s compat31

# ###########################################################################
# functions
# ###########################################################################
function my_exit ()
{
  
  if [ "${ERR_CNT}" == "0" ]; then
    echo "Test PASSED"
  else
    echo "Test FAILED!!! (ERR_CNT:${ERR_CNT})"
  fi
  
  echo "Exiting with ecode:$1"
  exit $1
}


function myexit ()
{
  local retcode=${ERR_CNT}
  [ -n "$1" ] && retcode=$1
  echo "===---=== SUMMARY ===---==="
  echo "dur:${SECONDS}, tst_cnt:${TST_CNT}, err_cnt:${retcode} (${ERR_CNT}), ecodes:${ECODES}"
  exit ${retcode}
}

# do_eval_ecode ( <exit-code> [expected-exit-code] )
#  exit code checking function
function do_eval_ecode ()
{
  local int_pass_same_ecode_as_got_ena=1
  local in_ecode=$1
  local in_exp_ecode=0
  [[ "$2" =~ "^[0-9]+|[Nn]one|[Tt]rue|[Ff]alse$" ]] && in_exp_ecode=$2
  # exit-code-expected may be:
  #   <number> ~ exit code         numbers should match
  #   None                         any exit-code is ok
  #   True                         exit-code of 0 ok
  #   False                        exit-code != 0 ok
  [ -z "${TST_CNT}" ] && TST_CNT=0
  [ -z "${ERR_CNT}" ] && ERR_CNT=0
  [ -z "${ECODES}" ] && ECODES=""

  let "TST_CNT++"

  if [[ "${in_exp_ecode}" =~ "^[0-9]+$" ]]; then
    [ "${in_ecode}" != "${in_exp_ecode}" ] && let "ERR_CNT++"
    ECODES="${ECODES}(${in_ecode})"
  elif [[ "${in_exp_ecode}" =~ "^[Nn]one$" ]]; then
    ECODES="${ECODES}[${in_ecode}]"
  elif [[ "${in_exp_ecode}" =~ "^[Tt]rue$" ]]; then
    [ "${in_ecode}" != "0" ] && let "ERR_CNT++"
    ECODES="${ECODES}(${in_ecode})"
  elif [[ "${in_exp_ecode}" =~ "^[Ff]alse$" ]]; then
    [ "${in_ecode}" == "0" ] && let "ERR_CNT++"
    ECODES="${ECODES}{${in_ecode}}"
  else
    ECODES="${ECODES}$?"
  fi

  # return exit code
  if [ "${int_pass_same_ecode_as_got_ena}" == '1' ]; then
    return ${in_ecode}
  else
    return 0
  fi
}


# gdb_bt( pid )
# creates gdb full backtrace on pid process, exports file in GDB_FEED_FN
# ###########################################################################
function gdb_bt ()
{
  GDB_FEED_FN="./gdb.in"
  echo "attach $1" > ${GDB_FEED_FN}
  echo "thread apply all bt" >> ${GDB_FEED_FN}
  gdb < ${GDB_FEED_FN}
}

# ###########################################################################
# mrg_gen_my_rand_in_range ( [min], [max] )
# mrg_rand                 ( [min], [max] )
#   generate custom ranged random number:
#     mrg_gen_my_rand_in_range      interval <0...32767> /like in bash/
#     mrg_gen_my_rand_in_range M    interval <0...M)
#     mrg_gen_my_rand_in_range m M  interval <m...M)
#   output random number is stored in MY_RAND environment variable
# 
# example:
#   $ mrg_rand 7 45 ; echo "ecode:$?, result:${MY_RAND}"
#   ecode:0, result:26
# ###########################################################################
function mrg_gen_my_rand_in_range ()
{
  local bash_rand_max=32767
  
  if [ "$#" == "0" ]; then
    # no range specified, return the raw rand as provided by bash
    MY_RAND=${RANDOM}
  else
    if [[ "$1" =~ "[0-9]+" ]]; then
      if [ -z "$2" ]; then
        # <0...M) range specified
        let "MY_RAND=($1*${RANDOM})/${bash_rand_max}"
      else
        if [[ "$2" =~ "[0-9]+" ]]; then
          # detect whether parameters are min , max
          if [ "$1" -le "$2" ]; then
            # <m...M) range specified
            let "MY_RAND=((($2-$1)*${RANDOM})/${bash_rand_max})+$1"
          else
            # <m...M) range specified, but max, min
            let "MY_RAND=((($1-$2)*${RANDOM})/${bash_rand_max})+$2"
          fi
        else
          # <0...M) range specified, because of fact that $2 is not number
          let "MY_RAND=($1*${RANDOM})/${bash_rand_max}"
          return 1
        fi
      fi
    else
      # error exit
      return 1
    fi
  fi
  
  # correct exit
  return 0
}

# function aliases
function mrg_rand ()
{
  mrg_gen_my_rand_in_range $1 $2
  return $?
}

# ###########################################################################
# mrg_gen_my_rand_str ( [str_length] , [regexp_filter])
# mrg_rand_str        ( [str_length] , [regexp_filter])
#   generate custom random string of length specified as str_length (10)
#   by default only [0-9a-f] characters are allowed, but can be specified as
#   regexp_filter
#   output random string is stored in environment variable MY_RAND_STR
# 
# example:
#   $ mrg_rand_str 57 [0-9a-zA-Z_/] ; echo "ecode:$?, result:${MY_RAND_STR}"
#   ecode:0, result:F0hkfP/sKZiAk3HmTVAcww/HAN/q8l3ecVJGMBdlIBsXk4zeCh2qg1u17
# ###########################################################################
function mrg_gen_my_rand_str ()
{
  
  local int_str=""                     # internal string - init assignment
  
  # assign string length from $1
  if [[ -z "$1" || "$1" < "1" || "$1" =~ "[^0-9]+" ]]; then
    local str_len=10                   # string length
  else
    local str_len=$1                   # string length
  fi
  local max_iterations=$((100*str_len)) # maximum of iterations
  
  # assign filter from $2
  if [[ -z "$2" ]]; then
    local str_regexp="[0-9a-zA-Z]"
  else
    local str_regexp=$2
  fi
  
  # main loop
  while [ 1 ]; do
    # generate random number
    #mrg_gen_my_rand_in_range 32 128
    local bash_rand_max=32767
    let "my_rand=(((128-32)*${RANDOM})/${bash_rand_max})+32"
    # convert number to character
    local int_char="$(echo "${my_rand}" | awk '{printf("%c",$1)}')"
    # apply filtering and possible append
    if [[ "${int_char}" =~ "${str_regexp}" ]]; then
      int_str="${int_str}${int_char}"
    fi
    # break the loop if we have all expected characters
    if [ "$(echo ${int_str} | \
            awk '{printf(length($1))}')" == "${str_len}" ]; then
      break
    fi
    
    # if maximum of iterations is reached then break the loop
    let "max_iterations--"
    if [ "${max_iterations}" == "0" ]; then
      break
    fi
  done
  
  if [ "$(echo ${int_str} | \
          awk '{printf(length($1))}')" == "${str_len}"  ]; then
    # assign output value
    MY_RAND_STR=${int_str}
    return 0
  else
    echo "mrg_gen_my_rand_str: Cannot generate random string. Most probably malformed regexp filter (${str_regexp})"
    return 1
  fi
  
}

# function aliases
function mrg_rand_str ()
{
  mrg_gen_my_rand_str $1 $2
  return $?
}


# gen_certs [ssl-dir]
function gen_certs ()
{
  rpm -q nss-tools || yum install -y nss-tools
  rpm -q nss-tools || return 1
  
  local dirname='/var/lib/qpidd/'
  [ -n "$1" -a -d "$1" ] && dirname=$1
  
  echo password > ${dirname}/passwordfile
  #chown qpidd:qpidd ${dirname}/passwordfile
  #chmod 600 ${dirname}/passwordfile
  cd ${dirname}
  certutil -N -d . -f passwordfile
  certutil -S -d . -f passwordfile -n $(hostname) -s "CN=$(hostname)"  -t "CT,," -x -z /usr/bin/certutil
  #certutil -S -d . -f passwordfile -n nickname -s "CN=nickname"  -t "CT,," -x -z /usr/bin/certutil
  cd -
  
  echo ${dirname} ${dirname}/passwordfile $(hostname) $(hostname)
  
}

# gen_sasldb [sasl-db] [user] [realm]
function gen_sasldb ()
{
 
  rpm -q cyrus-sasl || yum install -y cyrus-sasl
  rpm -q cyrus-sasl || return 1
  
  local sasl_db='qpidd.sasldb'
  local dirname='/var/lib/qpidd/'
  [ -n "$1" -a -d "$1" ] && dirname=$1
  local user='guest'
  [ -n "$2" ] && user=$2
  local realm='QPID'
  [ -n "$3" ] && realm=$3
  
  
  cd ${dirname}
  sasldblistusers2 -f ${sasl_db} | grep ${user} | grep ${realm}
  if [ $? != 0 ]; then
    touch ${sasl_db}
    echo ${user} | saslpasswd2 -p -f ${sasl_db} -u ${realm} ${user}
    sasldblistusers2 -f ${sasl_db}
    chown qpidd:qpidd ${sasl_db}
  fi
  cd -

  echo "${dirname} ${sasl_db} ${user} ${realm}"

}


# lf_gen_ssl_certs ( [cert-db-dir] )
# function to generate c++ broker / c++ client certificates
function lf_gen_ssl_certs() {
  MRG_CA_DIR="$1"
  #CA authority
  [ -z "${MRG_CA_DIR}" ] && \
    export MRG_CA_DIR="$(readlink -f $(dirname $0))/CA_db"
  [ -z "${MRG_CA_PASSWORD}" ] && \
    export MRG_CA_PASSWORD='imstrong'
  [ -z "${MRG_CA_CN}" ] && \
    export MRG_CA_CN='CAcert'
  
  local MRG_NOISE_FILE=$(mktemp)
  dd count=1 bs=4096 if=/dev/urandom of=${MRG_NOISE_FILE}

  MRG_CA_PW_FILE="${MRG_CA_DIR}/pswdfile"
  [ -z "${MRG_CA_NICK}" ] && \
    export MRG_CA_NICK='CAnick'
  
  # SERVER certification variables
  MRG_SRV_PASSWORD=${MRG_CA_PASSWORD}
  [ -z "${MRG_SRV_NICK}" ] && \
    export MRG_SRV_NICK=server_$(hostname)
  [ -z "${MRG_SRV_CN}" ] && \
    export MRG_SRV_CN=$(hostname)
  
  if [ -z "${MRG_SRV_CN}" ]; then
    echo "ERROR: FQDN not obtained"
    return 1
  fi
  
  # CLIENT certification variables
  MRG_CLI_PASSWORD=${MRG_CA_PASSWORD}
  [ -z "${MRG_CLI_NICK}" ] && \
    export MRG_CLI_NICK=client_$(hostname)
  [ -z "${MRG_CLI_CN}" ] && \
    export MRG_CLI_CN=c$(hostname)
  
  # prepare directories
  if [ -n "${MRG_CA_DIR}" ]; then
    rm -rf ${MRG_CA_DIR}
    mkdir -p ${MRG_CA_DIR}
  fi
  
  # store password
  echo ${MRG_CA_PASSWORD} > ${MRG_CA_PW_FILE}
  
  # Initialise CERT DB
  certutil -N -d ${MRG_CA_DIR} -f ${MRG_CA_PW_FILE} || return 2
  # Generate a new public and private key pair within a key database
  certutil -G -d ${MRG_CA_DIR} -f ${MRG_CA_PW_FILE} -z ${MRG_NOISE_FILE} || return 3
  
  # Create the self-signed Root CA certificate, specifying the subject name for the certificate.
  echo -e "y\n0\nn\n" | certutil -S -d ${MRG_CA_DIR} -n "${MRG_CA_NICK}" -s "CN=${MRG_CA_CN}"\
    -t "CT,," -x -m 1000 -v 120 -f ${MRG_CA_PW_FILE} -z ${MRG_NOISE_FILE} || return 4
  
  #Create Server cert
  certutil -S -n "${MRG_SRV_NICK}" -s "CN=${MRG_SRV_CN}" -c "${MRG_CA_NICK}" -t "u,u,u"\
    -m 1001 -v 120 -d ${MRG_CA_DIR} -f ${MRG_CA_PW_FILE} -z ${MRG_NOISE_FILE} || return 5
  
  #Create Client cert
  certutil -S -n "${MRG_CLI_NICK}" -s "CN=${MRG_CLI_CN}" -c "${MRG_CA_NICK}" -t "u,u,u"\
    -m 1002 -v 120 -d ${MRG_CA_DIR} -f ${MRG_CA_PW_FILE} -z ${MRG_NOISE_FILE} || return 6

  rm -f ${MRG_NOISE_FILE}
  
  echo ${MRG_CA_DIR} ${MRG_CA_PW_FILE} ${MRG_SRV_CN} ${MRG_SRV_NICK}\
       ${MRG_CA_DIR} ${MRG_CA_PW_FILE} ${MRG_CLI_CN} ${MRG_CLI_NICK}
  
  return 0
}


# gen_certs2 [ssl-dir] [cert-nick]
function gen_certs2 ()
{
  rpm -q nss-tools || yum install -y nss-tools
  rpm -q nss-tools || return 1
  
  local nick='CAnick'
  local dirname='/var/lib/qpidd/'
  [ -n "$1" -a -d "$1" ] && dirname=$1
  [ -n "$2" ] && nick=$2
  
  pushd ${dirname}

  local pswdfn="$(pwd)/pswdfile"
  echo imstrong > ${pswdfn}
  certutil -N -d $(pwd) -f ${pswdfn}
  certutil -S -d $(pwd) -f ${pswdfn} -n ${nick} -s "CN=$(hostname)"  -t "CT,," -x -z $(which certutil)
  #certutil -S -d . -f passwordfile -n nickname -s "CN=nickname"  -t "CT,," -x -z /usr/bin/certutil
  popd

  #certutil -L -d ${dirname}
  
  echo ${dirname} ${pswdfn} ${nick} $(hostname)
  
}



function do_cmd ()
{
  local ecode=124
  
  echo "  $1"
  if [ -n "${QPID_SSL_CERT_DB}" ]; then
    echo "$1" > /tmp/asx
    expect -f run.exp "bash /tmp/asx"
    ecode=$?
  else
    $1
    ecode=$?    
  fi

  return ${ecode}
}
          






#############################################################################

function mysync ()
{
  sync
  sleep 1
  sync
}

# attach_alert_receivers [host] [port]
function attach_alert_receivers ()
{
  local lpids=''
  local host=$(hostname)
  [ -n "$1" ] && host=$1
  local port=$2
  [ -n "$2" ] && port=$2
  
  for i_e in clientConnect clientDisconnect clientConnectFail; do
    # browse individual events
    
    # java
    ./event_receiver.sh "${host}:${port}" \
      "qmf.default.topic/agent.ind.event.org_apache_qpid_broker.${i_e}.#" &> java.${i_e}.log &
    lpids="${lpids} $!"
    # python
    python -u ./event_receiver.py "${host}:${port}" \
      "qmf.default.topic/agent.ind.event.org_apache_qpid_broker.${i_e}.#" &> python.${i_e}.log &
    lpids="${lpids} $!"
    # c++
    ./event_receiver "${host}:${port}" \
      "qmf.default.topic/agent.ind.event.org_apache_qpid_broker.${i_e}.#" &> cpp.${i_e}.log &
    lpids="${lpids} $!"
  
  done

  #ps ${lpids}
  
  echo ${lpids}

}

# truncate_alert_receivers_logs
function truncate_alert_receivers_logs ()
{
  for i_e in clientConnect clientDisconnect clientConnectFail; do
    :> cpp.${i_e}.log
    :> python.${i_e}.log
    :> java.${i_e}.log
  done
}



function check_alerts ()
{
  local filter=$1
  
  local ccnt=$2
  local dcnt=$3
  local fcnt=$4
  local cnow=$5
  local dnow=$6
  local fnow=$7

  local pname=''
  local pid=''
  local ppid=''

  [ -n "$8" ] && pname="$8"
  [ -n "$9" ] && pid="$9"
  [ -n "${10}" ] && ppid="${10}"

  local alerts=( clientConnect clientDisconnect clientConnectFail )
  local achecks=( ${cnow} ${dnow} ${fnow} )

  local err_cnt=0
  local err_str=''
  
  local i=
  local i_l=
  for i_l in cpp python java; do
    if [ "$(grep "^|" ${i_l}.clientConnect.log | egrep "${filter}" | wc -l)" != "${ccnt}" ]; then
      let 'err_cnt++'
      echo "$(grep "^|" ${i_l}.clientConnect.log | egrep "${filter}" | wc -l)x${ccnt}"
    fi
    
    err_str="${err_str} ${err_cnt}"
    [ "$(grep -c clientDisconnect ${i_l}.clientConnect.log)" != "0" ] && let 'err_cnt++'
    err_str="${err_str} ${err_cnt}"
    [ "$(grep -c clientConnectFail ${i_l}.clientConnect.log)" != "0" ] && let 'err_cnt++'
    err_str="${err_str} ${err_cnt}"
  done
  err_str="${err_str} |"
  for i_l in cpp python java; do
    if [ "$(grep "^|" ${i_l}.clientDisconnect.log | egrep "${filter}" | wc -l)" != "${dcnt}" ]; then
      let 'err_cnt++'
      echo "$(grep "^|" ${i_l}.clientDisconnect.log | egrep "${filter}" | wc -l)X${dcnt}"
    fi
    
    err_str="${err_str} ${err_cnt}"
    [ "$(grep -c clientConnect ${i_l}.clientDisconnect.log)" != "0" ] && let 'err_cnt++'
    err_str="${err_str} ${err_cnt}"
    [ "$(grep -c clientConnectFail ${i_l}.clientDisconnect.log)" != "0" ] && let 'err_cnt++'
    err_str="${err_str} ${err_cnt}"
  done
  err_str="${err_str} |"
  for i_l in cpp python java; do
    if [ "$(grep "^|" ${i_l}.clientConnectFail.log | egrep "${filter}" | wc -l)" != "${fcnt}" ]; then
      let 'err_cnt++'
      echo "$(grep "^|" ${i_l}.clientConnectFail.log | egrep "${filter}" | wc -l)x${fcnt}"
    fi
    err_str="${err_str} ${err_cnt}"
    [ "$(grep -c "clientConnect " ${i_l}.clientConnectFail.log)" != "0" ] && let 'err_cnt++'
    err_str="${err_str} ${err_cnt}"
    [ "$(grep -c clientDisconnect ${i_l}.clientConnectFail.log)" != "0" ] && let 'err_cnt++'
    err_str="${err_str} ${err_cnt}"
  done
  err_str="${err_str} |"

  if [ -n "${pname}" ]; then
    for ((i=0; i<${#alerts[*]}; i++  )); do
      # browse alerts
      if [ "${achecks[${i}]}" != "0" ]; then
        for i_l in cpp python java; do
          grep ${alerts[${i}]} ${i_l}.${alerts[${i}]}.log | tail -1| grep "${pname}" &>/dev/null || \
          let 'err_cnt++'
          err_str="${err_str} ${err_cnt}"
        done 
      fi
    done
  fi
  err_str="${err_str} |"
  
  if [ -n "${pid}" ]; then
    for ((i=0; i<${#alerts[*]}; i++  )); do
      # browse alerts
      if [ "${achecks[${i}]}" != "0" ]; then
        for i_l in cpp python java; do
          [ "$(grep ${alerts[${i}]} ${i_l}.${alerts[${i}]}.log | tail -1| awk '{printf $(NF-1)}')" != ${pid} ] && let 'err_cnt++'
          err_str="${err_str} ${err_cnt}"
        done 
      fi
    done
  fi
  err_str="${err_str} |"
  
  if [ -n "${ppid}" ]; then
    for ((i=0; i<${#alerts[*]}; i++  )); do
      # browse alerts
      if [ "${achecks[${i}]}" != "0" ]; then
        for i_l in cpp python java; do
          if [ "$(grep ${alerts[${i}]} ${i_l}.${alerts[${i}]}.log | tail -1| awk '{printf $(NF)}')" != ${ppid} ]; then
            echo "$(grep ${alerts[${i}]} ${i_l}.${alerts[${i}]}.log | tail -1| awk '{printf $(NF)}') x ${ppid}"
            let 'err_cnt++'
          fi 

          
        done 
      fi
    done
    err_str="${err_str} ${err_cnt}"
  fi
  
  echo ${err_str}
  
  return ${err_cnt}

}


function f_abs ()
{
  echo "$(echo $* | tr -d -c '0-9')"
}











# eof