#!/bin/bash
#
# Usage:
# runtest772028
# 
# by freznice, pematous, crolke

shopt | grep compat31 >/dev/null && shopt -s compat31

ERR_CNT=0
TST_CNT=0
ECODES=''

IP=$(ip -4 r l | grep eth0 | grep src | awk '{print $NF}')
BZNAME=BZ772028
CNAME=${BZNAME}_${IP}

if [ "system" -eq "system" ]; then
  # using installed packages
  QSEND=$(rpm -ql rh-qpid-cpp-tests | grep qpid-send)
  QRECEIVE=$(rpm -ql rh-qpid-cpp-tests | grep qpid-receive)
  DRAIN=$(rpm -ql python-qpid | grep '/drain')
  [ -z "${QSEND}" -o -z "${QRECEIVE}" ] &&  exit 1
  QSTAT=$(rpm -ql qpid-tools | grep '/qpid-stat')
  QCONFIG=$(rpm -ql qpid-tools | grep '/qpid-config')
  QROUTE=$(rpm -ql qpid-tools | grep '/qpid-route')
  BROKER_FN=$(rpm -ql qpid-cpp-server | grep 'in/qpidd$')
else
  # using crolke local build
  QSEND=/home/crolke/git/rh-qpid/build/src/tests/qpid-send
  QRECEIVE=/home/crolke/git/rh-qpid/build/src/tests/qpid-receive
  DRAIN=/home/crolke/git/rh-qpid/build/src/../../qpid/python/examples/api/drain
  [ -z "${QSEND}" -o -z "${QRECEIVE}" ] &&  exit 1
  QSTAT=/home/crolke/git/rh-qpid/build/src/../../qpid/tools/src/py/qpid-stat
  QCONFIG=/home/crolke/git/rh-qpid/build/src/../../qpid/tools/src/py/qpid-config
  QROUTE=/home/crolke/git/rh-qpid/build/src/../../qpid/tools/src/py/qpid-route
  BROKER_FN=/home/crolke/git/rh-qpid/build/src/qpidd
  export PYTHONPATH=/home/crolke/git/rh-qpid/qpid/tools/src/py:/home/crolke/git/rh-qpid/qpid/python
fi

BROKER_NAME=$(basename ${BROKER_FN})
BROKER_EXTERNAL_SWITCHES='--log-enable info+'

echo "Configuration: ------------------------[begin]"
set | egrep '^Q[A-Z]+|^BROKER_|^DRAIN'
echo "Configuration: --------------------------[end]"

echo "Packages: -----------------------------[begin]"
rpm -qa | grep qpid | sort -u
echo "Packages: -------------------------------[end]"

# this script accepts no args from the command line

PORTS_OFFSET=5672
DPORT=5999
CLUSTER_CNT=1

QLL=info+
QLL=debug+
#QLL=trace+

QTCPPORTS=
for ((i=0; i< ${CLUSTER_CNT}; i++)); do
  QTCPPORTS="${QTCPPORTS} $(( ${PORTS_OFFSET} + ${i} ))"
done
QTCPPORTS=( ${QTCPPORTS} )


# source functions
source ./functions.sh


# state_check 
function state_check ()
{
    echo "null state check"
}


# clean-up from past
killall ${BROKER_NAME}

ulimit -c unlimited
service iptables stop
setenforce 0

rm -f *.log core.*
rm -rf ./567? ./${DPORT}



QSASLUSER=guest

## make the clients
#make clean
#make
#do_eval_ecode $? True || myexit


ssldbdir='/var/lib/qpidd'
# ###########################################################################
# prepare SASL db
# ###########################################################################
#echo "Generation of SASL user database:"
#
#vals=$(gen_sasldb /var/lib/qpidd guest QPID | tail -1)
#ecode=$?
#QSASLDIR="$(echo ${vals} | awk '{print $1}')"
#QSASLFILE="$(echo ${vals} | awk '{print $2}')"
#QSASLUSER="$(echo ${vals} | awk '{print $3}')"
#QSASLREALM="$(echo ${vals} | awk '{print $4}')"
#echo ".done:${ecode} ${QSASLDIR},${QSASLFILE},${QSASLUSER},${QSASLREALM}"




# ###########################################################################
# launch broker / cluster
# ###########################################################################
qpidd_cp=" --auth=no --log-enable=${QLL}"
qpidd_cl_cp=""
echo "starting standalone broker:"


for ((i=0; i<${CLUSTER_CNT}; i++)); do
  
  tcpp=${QTCPPORTS[${i}]}
  
  # delete data-dir (if exists)
  rm -rf ${tcpp}
  mkdir ${tcpp}
  
  ##vals=$(gen_sasldb ${PWD}/${tcpp} guest QPID | tail -1)
  ecode=$?
  
  echo "  ${BROKER_FN} -d ${BROKER_EXTERNAL_SWITCHES} -p ${tcpp} ${qpidd_cp} ${qpidd_cl_cp} --data-dir ${PWD}/${tcpp} (${ecode}:${vals})"
  ${BROKER_FN} -d ${BROKER_EXTERNAL_SWITCHES} -p ${tcpp} ${qpidd_cp} ${qpidd_cl_cp} --data-dir ${PWD}/${tcpp} --log-to-file ${PWD}/${tcpp}/qpidd.log > ${PWD}/${tcpp}/qpidd_init.log 2>&1
  sleep 5
  echo -n "."
done

DBRK_CNT=0

echo ".done"


sleep 3
netstat -nlp | grep ${BROKER_NAME}
sleep 3


# ###########################################################################
# broker instance[s] check
# ###########################################################################
dpid=$(netstat -nlp | grep ${BROKER_NAME} | awk '{print $NF}' | grep ${DPORT} | sort -u | awk  -F/ '{printf $(NF-1) " "}')
pids=$(netstat -nlp | grep ${BROKER_NAME} | awk '{print $NF}' | sort -u | awk  -F/ '{printf $(NF-1) " "}')
pids_cnt=$(netstat -nlp | grep ${BROKER_NAME} | awk '{print $NF}' | sort -u | wc -l)

if [ -z "${pids}" -o "${pids_cnt}" != "$(( ${CLUSTER_CNT} + ${DBRK_CNT}))" ]; then
  echo -n "broker[s] not running! "
  kill ${pids}
else
  echo -n "broker[s] running "
fi
echo "(pids:${pids}, pids_cnt:${pids_cnt}, cluster_cnt:${CLUSTER_CNT}, dest-brk-cnt:${DBRK_CNT})"

test -n "${pids}"
do_eval_ecode $? True || myexit
[ "${pids_cnt}" == "$(( ${CLUSTER_CNT} + ${DBRK_CNT}))" ]
do_eval_ecode $? True || myexit


# ###########################################################################
# core test - looping the tsxtest with load using perftest
# ###########################################################################
echo "Core tests: [${TST_CNT}/${ERR_CNT}]"
# check the brokers using qpid-stat
${QSTAT} -g | egrep "standalone|${CNAME}"
do_eval_ecode $?

# Emit the python code script
#
# bz772028.py - reproducer test code from the bz by Petr Matousek
#
# On a bad system there is an error message after the print statement
# On a good system there are no error messages.
# 
cat > runtest.py <<EOF
from qpid.messaging import *
connection = Connection("localhost:5672")
connection.open()
session = connection.session()
receiver = session.receiver("q;{create:receiver,delete:receiver}")
print receiver.unsettled()
connection.close()
EOF

python runtest.py
do_eval_ecode $? True || myexit

echo "end of code test [${TST_CNT}/${ERR_CNT}]"

# ###########################################################################
# broker instance[s] check
# ###########################################################################
echo "Final broker[s] test: [${TST_CNT}/${ERR_CNT}]"

for i_pid in ${pids}; do
  netstat -nlp | grep ${BROKER_NAME} | grep ${i_pid}
  do_eval_ecode $? True || echo ".ERROR: broker pid:${i_pid} not running"
done
echo "done."

sleep 5

# ###########################################################################
# broker instance[s] stop
# ###########################################################################
echo "Stopping brokers... [${TST_CNT}/${ERR_CNT}]"
for ((i=$(( ${CLUSTER_CNT} - 1 )); i>=0; i--)); do
  
  tcpp=${QTCPPORTS[${i}]}
  
  ${BROKER_FN} -p ${tcpp} -q
  do_eval_ecode $?
  echo -n "."
  sleep 1
done

if [ "${DBRK_CNT}" -gt 0 ]; then
  ${BROKER_FN} -p ${DPORT} -q
  do_eval_ecode $?
  echo -n 'd'
fi
echo ''

killall ${BROKER_NAME} &>/dev/null

## ?epid not defined?: ps ${epid} &>/dev/null && kill ${epid}

echo "Broker log checks... [${TST_CNT}/${ERR_CNT}]"
awk 'BEGIN{c=0}{if($4 ~ /error|critical/){printf("%s: %s\n",FILENAME, $0);c++}}END{exit(c)}' 5*/qpidd.log
do_eval_ecode $? || echo "ERROR: broker log files highlights problems!"

echo "Checking core files... [${TST_CNT}/${ERR_CNT}]"
[ $(ls ./core.* 2>/dev/null | wc -l) == '0' ]
do_eval_ecode $?


# exit summary
myexit

# eof
